@@ -14,10 +14,6 @@ from trytond.transaction import Transaction
 from trytond.pool import Pool
 from trytond.tools import decistmt
 
-__all__ = ['MoveTemplate', 'MoveTemplateKeyword',
-    'MoveLineTemplate', 'TaxLineTemplate',
-    'CreateMove', 'CreateMoveTemplate', 'CreateMoveKeywords']
-
 
 class MoveTemplate(DeactivableMixin, ModelSQL, ModelView):
     'Account Move Template'
@@ -26,7 +22,12 @@ class MoveTemplate(DeactivableMixin, ModelSQL, ModelView):
     keywords = fields.One2Many('account.move.template.keyword', 'move',
         'Keywords')
     company = fields.Many2One('company.company', 'Company', required=True)
-    journal = fields.Many2One('account.journal', 'Journal', required=True)
+    journal = fields.Many2One(
+        'account.journal', 'Journal', required=True,
+        context={
+            'company': Eval('company', -1),
+            },
+        depends=['company'])
     date = fields.Char('Date', help='Leave empty for today.')
     description = fields.Char('Description',
         help="Keyword values substitutions are identified "
@@ -78,6 +79,11 @@ class MoveTemplateKeyword(sequence_ordered(), ModelSQL, ModelView):
             'required': Eval('type_') == 'numeric',
             }, depends=['type_'])
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('move')
+
     @staticmethod
     def default_required():
         return False
@@ -159,7 +165,8 @@ class MoveLineTemplate(ModelSQL, ModelView):
         help="A python expression that will be evaluated with the keywords.")
     account = fields.Many2One('account.account', 'Account', required=True,
         domain=[
-            ('kind', '!=', 'view'),
+            ('type', '!=', None),
+            ('closed', '!=', True),
             ('company', '=', Eval('_parent_move', {}).get('company', -1)),
             ])
     party = fields.Char('Party',
@@ -176,6 +183,11 @@ class MoveLineTemplate(ModelSQL, ModelView):
         "by braces ('{' and '}').")
     taxes = fields.One2Many('account.tax.line.template', 'line', 'Taxes')
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('move')
+
     @fields.depends('account')
     def on_change_with_party_required(self, name=None):
         if self.account:
@@ -224,6 +236,11 @@ class TaxLineTemplate(ModelSQL, ModelView):
                     ).get('_parent_move', {}).get('company', -1)),
             ])
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('line')
+
     @classmethod
     def __register__(cls, module_name):
         super(TaxLineTemplate, cls).__register__(module_name)
@@ -306,10 +323,9 @@ class CreateMove(Wizard):
 
     def transition_start(self):
         context = Transaction().context
-        model = context.get('active_model')
         action_id = context.get('action_id')
         period = context.get('period')
-        if model == 'account.move.line':
+        if self.model and self.model.__name__ == 'account.move.line':
             # Template id is used as action
             self.template.template = action_id
             self.template.period = period
@@ -318,8 +334,7 @@ class CreateMove(Wizard):
             return 'template'
 
     def transition_create_(self):
-        model = Transaction().context.get('active_model')
-        if model == 'account.move.line':
+        if self.model and self.model.__name__ == 'account.move.line':
             self.create_move()
             return 'end'
         else:
@@ -331,8 +346,7 @@ class CreateMove(Wizard):
         return action, {}
 
     def end(self):
-        model = Transaction().context.get('active_model')
-        if model == 'account.move.line':
+        if self.model and self.model.__name__ == 'account.move.line':
             return 'reload'
 
 
@@ -361,4 +375,5 @@ class CreateMoveTemplate(ModelView):
 
 class CreateMoveKeywords(ModelView):
     'Create Move from Template'
+    __no_slots__ = True
     __name__ = 'account.move.template.create.keywords'
