@@ -9,26 +9,20 @@ from sql import Literal
 from sql.aggregate import Sum
 from sql.conditionals import Case
 
+from trytond.i18n import gettext
 from trytond.model import (
     ModelView, ModelSQL, MatchMixin, DeactivableMixin, fields,
     sequence_ordered, tree)
+from trytond.model.exceptions import AccessError
 from trytond.wizard import Wizard, StateView, StateAction, Button
 from trytond import backend
 from trytond.pyson import Eval, If, Bool, PYSONEncoder
 from trytond.transaction import Transaction
-from trytond.tools import cursor_dict
+from trytond.tools import cursor_dict, lstrip_wildcard
 from trytond.pool import Pool
 
 from .common import PeriodMixin, ActivePeriodMixin
 
-__all__ = ['TaxGroup', 'TaxCodeTemplate', 'TaxCode',
-    'TaxCodeLineTemplate', 'TaxCodeLine',
-    'OpenChartTaxCodeStart', 'OpenChartTaxCode',
-    'TaxTemplate', 'Tax', 'TaxLine', 'TaxRuleTemplate', 'TaxRule',
-    'TaxRuleLineTemplate', 'TaxRuleLine',
-    'OpenTaxCode',
-    'TestTax', 'TestTaxView', 'TestTaxViewResult']
-
 KINDS = [
     ('sale', 'Sale'),
     ('purchase', 'Purchase'),
@@ -141,6 +135,8 @@ class TaxCode(ActivePeriodMixin, tree(), ModelSQL, ModelView):
         'account.tax.code', 'Parent', select=True, states=_states)
     lines = fields.One2Many('account.tax.code.line', 'code', "Lines")
     childs = fields.One2Many('account.tax.code', 'parent', 'Children')
+    currency = fields.Function(fields.Many2One('currency.currency',
+        'Currency'), 'on_change_with_currency')
     currency_digits = fields.Function(fields.Integer('Currency Digits'),
         'on_change_with_currency_digits')
     amount = fields.Function(fields.Numeric(
@@ -176,9 +172,14 @@ class TaxCode(ActivePeriodMixin, tree(), ModelSQL, ModelView):
         return False
 
     @fields.depends('company')
-    def on_change_with_currency_digits(self, name=None):
+    def on_change_with_currency(self, name=None):
         if self.company:
-            return self.company.currency.digits
+            return self.company.currency.id
+
+    @fields.depends('currency')
+    def on_change_with_currency_digits(self, name=None):
+        if self.currency:
+            return self.currency.digits
         return 2
 
     @classmethod
@@ -225,8 +226,11 @@ class TaxCode(ActivePeriodMixin, tree(), ModelSQL, ModelView):
             bool_op = 'AND'
         else:
             bool_op = 'OR'
+        code_value = clause[2]
+        if clause[1].endswith('like'):
+            code_value = lstrip_wildcard(clause[2])
         return [bool_op,
-            ('code',) + tuple(clause[1:]),
+            ('code', clause[1], code_value) + tuple(clause[3:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
 
@@ -295,6 +299,11 @@ class TaxCodeLineTemplate(ModelSQL, ModelView):
             ('credit', "Credit"),
             ], "Type", required=True)
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('code')
+
     def _get_tax_code_line_value(self, line=None):
         value = {}
         for name in ['operator', 'amount', 'type']:
@@ -367,6 +376,11 @@ class TaxCodeLine(ModelSQL, ModelView):
 
     del _states
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.__access__.add('code')
+
     @classmethod
     def default_operator(cls):
         return '+'
@@ -386,9 +400,27 @@ class TaxCodeLine(ModelSQL, ModelView):
             ('type', '=', self.amount),
             ]
         if self.type == 'invoice':
-            domain.append(('amount', '>', 0))
+            domain.append(['OR',
+                    [('amount', '>', 0), ['OR',
+                            ('move_line.debit', '>', 0),
+                            ('move_line.credit', '>', 0),
+                            ]],
+                    [('amount', '<', 0), ['OR',
+                            ('move_line.debit', '<', 0),
+                            ('move_line.credit', '<', 0),
+                            ]],
+                    ])
         elif self.type == 'credit':
-            domain.append(('amount', '<', 0))
+            domain.append(['OR',
+                    [('amount', '<', 0), ['OR',
+                            ('move_line.debit', '>', 0),
+                            ('move_line.credit', '>', 0),
+                            ]],
+                    [('amount', '>', 0), ['OR',
+                            ('move_line.debit', '<', 0),
+                            ('move_line.credit', '<', 0),
+                            ]],
+                    ])
         return domain
 
     @classmethod
@@ -424,14 +456,14 @@ class OpenChartTaxCodeStart(ModelView):
         ('periods', 'By Periods'),
         ], 'Method', required=True)
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
-        help='Leave empty for all open fiscal year',
+        help='Leave empty for all open fiscal year.',
         states={
             'invisible': Eval('method') != 'fiscalyear',
             'required': Eval('method') == 'fiscalyear',
             },
         depends=['method'])
     periods = fields.Many2Many('account.period', None, None, 'Periods',
-        help='Leave empty for all periods of all open fiscal year',
+        help='Leave empty for all periods of all open fiscal year.',
         states={
             'invisible': Eval('method') != 'periods',
             'required': Eval('method') == 'periods',
@@ -473,12 +505,17 @@ class OpenChartTaxCode(Wizard):
         return 'end'
 
 
-class TaxTemplate(sequence_ordered(), ModelSQL, ModelView):
+class TaxTemplate(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
     'Account Tax Template'
     __name__ = 'account.tax.template'
     name = fields.Char('Name', required=True)
     description = fields.Char('Description', required=True)
-    group = fields.Many2One('account.tax.group', 'Group')
+    group = fields.Many2One(
+        'account.tax.group', 'Group',
+        states={
+            'invisible': Bool(Eval('parent')),
+            },
+        depends=['parent'])
     start_date = fields.Date('Starting Date')
     end_date = fields.Date('Ending Date')
     amount = fields.Numeric('Amount', digits=(16, 8),
@@ -504,10 +541,28 @@ class TaxTemplate(sequence_ordered(), ModelSQL, ModelView):
         depends=['parent'])
     parent = fields.Many2One('account.tax.template', 'Parent')
     childs = fields.One2Many('account.tax.template', 'parent', 'Children')
-    invoice_account = fields.Many2One('account.account.template',
-            'Invoice Account')
-    credit_note_account = fields.Many2One('account.account.template',
-            'Credit Note Account')
+    invoice_account = fields.Many2One(
+        'account.account.template', 'Invoice Account',
+        domain=[
+            ('type.statement', '=', 'balance'),
+            ('closed', '!=', True),
+            ],
+        states={
+            'invisible': Eval('type') == 'none',
+            'required': Eval('type') != 'none',
+            },
+        depends=['type'])
+    credit_note_account = fields.Many2One(
+        'account.account.template', 'Credit Note Account',
+        domain=[
+            ('type.statement', '=', 'balance'),
+            ('closed', '!=', True),
+            ],
+        states={
+            'invisible': Eval('type') == 'none',
+            'required': Eval('type') != 'none',
+            },
+        depends=['type'])
     account = fields.Many2One('account.account.template', 'Account Template',
             domain=[('parent', '=', None)], required=True)
     legal_notice = fields.Text("Legal Notice")
@@ -516,10 +571,6 @@ class TaxTemplate(sequence_ordered(), ModelSQL, ModelView):
     def __setup__(cls):
         super(TaxTemplate, cls).__setup__()
         cls._order.insert(0, ('account', 'ASC'))
-        cls._error_messages.update({
-                'update_unit_price_with_parent': ('"Update Unit Price" can '
-                    'not be set on tax "%(template)s" which has a parent.'),
-                })
 
     @classmethod
     def validate(cls, tax_templates):
@@ -529,9 +580,9 @@ class TaxTemplate(sequence_ordered(), ModelSQL, ModelView):
 
     def check_update_unit_price(self):
         if self.update_unit_price and self.parent:
-            self.raise_user_error('update_unit_price_with_parent', {
-                    'template': self.rec_name,
-                    })
+            raise AccessError(
+                gettext('account.msg_tax_update_unit_price_with_parent',
+                    tax=self.rec_name))
 
     @staticmethod
     def default_type():
@@ -548,7 +599,7 @@ class TaxTemplate(sequence_ordered(), ModelSQL, ModelView):
         res = {}
         for field in ['name', 'description', 'sequence', 'amount', 'rate',
                 'type', 'start_date', 'end_date', 'update_unit_price',
-                'legal_notice']:
+                'legal_notice', 'active']:
             if not tax or getattr(tax, field) != getattr(self, field):
                 res[field] = getattr(self, field)
         for field in ('group',):
@@ -634,7 +685,7 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
         }
     name = fields.Char('Name', required=True, states=_states)
     description = fields.Char('Description', required=True, translate=True,
-            help="The name that will be used in reports", states=_states)
+            help="The name that will be used in reports.", states=_states)
     group = fields.Many2One('account.tax.group', 'Group',
         states={
             'invisible': Bool(Eval('parent')),
@@ -647,7 +698,7 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
             'required': Eval('type') == 'fixed',
             'invisible': Eval('type') != 'fixed',
             'readonly': _states['readonly'],
-            }, help='In company\'s currency',
+            }, help='In company\'s currency.',
         depends=['type'])
     rate = fields.Numeric('Rate', digits=(14, 10),
         states={
@@ -667,19 +718,17 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
             },
         depends=['parent'],
         help=('If checked then the unit price for further tax computation will'
-            ' be modified by this tax'))
+            ' be modified by this tax.'))
     parent = fields.Many2One('account.tax', 'Parent', ondelete='CASCADE',
         states=_states)
     childs = fields.One2Many('account.tax', 'parent', 'Children')
-    company = fields.Many2One('company.company', 'Company', required=True,
-        domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ], select=True)
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True)
     invoice_account = fields.Many2One('account.account', 'Invoice Account',
         domain=[
             ('company', '=', Eval('company')),
-            ('kind', 'not in', ['view', 'receivable', 'payable']),
+            ('type.statement', '=', 'balance'),
+            ('closed', '!=', True),
             ],
         states={
             'readonly': _states['readonly'],
@@ -691,7 +740,8 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
         'Credit Note Account',
         domain=[
             ('company', '=', Eval('company')),
-            ('kind', 'not in', ['view', 'receivable', 'payable']),
+            ('type.statement', '=', 'balance'),
+            ('closed', '!=', True),
             ],
         states={
             'readonly': _states['readonly'],
@@ -701,7 +751,8 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
         depends=['company', 'type'])
     legal_notice = fields.Text("Legal Notice", translate=True,
         states=_states)
-    template = fields.Many2One('account.tax.template', 'Template')
+    template = fields.Many2One('account.tax.template', 'Template',
+        ondelete='RESTRICT')
     template_override = fields.Boolean('Override Template',
         help="Check to override template definition",
         states={
@@ -731,14 +782,6 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
 
     del _states
 
-    @classmethod
-    def __setup__(cls):
-        super(Tax, cls).__setup__()
-        cls._error_messages.update({
-                'update_unit_price_with_parent': ('"Update Unit Price" can '
-                    'not be set on tax "%(template)s" which has a parent.'),
-                })
-
     @classmethod
     def validate(cls, taxes):
         super(Tax, cls).validate(taxes)
@@ -747,9 +790,9 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
 
     def check_update_unit_price(self):
         if self.parent and self.update_unit_price:
-            self.raise_user_error('update_unit_price_with_parent', {
-                    'tax': self.rec_name,
-                    })
+            raise AccessError(
+                gettext('account.msg_tax_update_unit_price_with_parent',
+                    tax=self.rec_name))
 
     @staticmethod
     def default_type():
@@ -786,21 +829,33 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
 
         columns = []
         amount = tax_line.amount
-        if backend.name() == 'sqlite':
+        debit = move_line.debit
+        credit = move_line.credit
+        if backend.name == 'sqlite':
             amount = TaxLine.amount.sql_cast(tax_line.amount)
+            debit = MoveLine.debit.sql_cast(debit)
+            credit = MoveLine.credit.sql_cast(credit)
+        is_invoice = (
+            ((amount > 0) & ((debit > 0) | (credit > 0)))
+            | ((amount < 0) & ((debit < 0) | (credit < 0)))
+            )
+        is_credit = (
+            ((amount < 0) & ((debit > 0) | (credit > 0)))
+            | ((amount > 0) & ((debit < 0) | (credit < 0)))
+            )
         for name, clause in [
                 ('invoice_base_amount',
-                    (amount > 0) & (tax_line.type == 'base')),
+                    is_invoice & (tax_line.type == 'base')),
                 ('invoice_tax_amount',
-                    (amount > 0) & (tax_line.type == 'tax')),
+                    is_invoice & (tax_line.type == 'tax')),
                 ('credit_base_amount',
-                    (amount < 0) & (tax_line.type == 'base')),
+                    is_credit & (tax_line.type == 'base')),
                 ('credit_tax_amount',
-                    (amount < 0) & (tax_line.type == 'tax')),
+                    is_credit & (tax_line.type == 'tax')),
                 ]:
             if name not in names:
                 continue
-            if backend.name() == 'postgresql':  # FIXME
+            if backend.name == 'postgresql':  # FIXME
                 columns.append(Sum(amount, filter_=clause).as_(name))
             else:
                 columns.append(Sum(Case([clause, amount])).as_(name))
@@ -884,15 +939,18 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
             for tax in group_taxes:
                 start_date = tax.start_date or datetime.date.min
                 end_date = tax.end_date or datetime.date.max
+                values = []
                 if not (start_date <= date <= end_date):
                     continue
                 if tax.type != 'none':
-                    value = tax._process_tax(price_unit)
-                    res.append(value)
-                    if tax.update_unit_price:
-                        unit_price_variation += value['amount']
+                    values.append(tax._process_tax(price_unit))
                 if len(tax.childs):
-                    res.extend(cls._unit_compute(tax.childs, price_unit, date))
+                    values.extend(
+                        cls._unit_compute(tax.childs, price_unit, date))
+                if tax.update_unit_price:
+                    for value in values:
+                        unit_price_variation += value['amount']
+                res.extend(values)
             price_unit += unit_price_variation
         return res
 
@@ -1009,8 +1067,8 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
                         vals = child.template._get_tax_value(tax=child)
                         invoice_account_id = (child.invoice_account.id
                             if child.invoice_account else None)
-                        if (child.template.invoice_account and
-                                invoice_account_id != template2account.get(
+                        if (child.template.invoice_account
+                                and invoice_account_id != template2account.get(
                                         child.template.invoice_account.id)):
                             vals['invoice_account'] = template2account.get(
                                 child.template.invoice_account.id)
@@ -1019,8 +1077,9 @@ class Tax(sequence_ordered(), ModelSQL, ModelView, DeactivableMixin):
                             vals['invoice_account'] = None
                         credit_note_account_id = (child.credit_note_account.id
                             if child.credit_note_account else None)
-                        if (child.template.credit_note_account and
-                                credit_note_account_id != template2account.get(
+                        if (child.template.credit_note_account
+                                and credit_note_account_id
+                                != template2account.get(
                                     child.template.credit_note_account.id)):
                             vals['credit_note_account'] = template2account.get(
                                 child.template.credit_note_account.id)
@@ -1042,7 +1101,7 @@ class _TaxKey(dict):
         self.update(kwargs)
 
     def _key(self):
-        return (self['account'], self['tax'])
+        return (self['account'], self['tax'], self['base'] >= 0)
 
     def __eq__(self, other):
         if isinstance(other, _TaxKey):
@@ -1053,10 +1112,12 @@ class _TaxKey(dict):
         return hash(self._key())
 
 
-_TaxableLine = namedtuple('_TaxableLine', ('taxes', 'unit_price', 'quantity'))
+_TaxableLine = namedtuple(
+    '_TaxableLine', ('taxes', 'unit_price', 'quantity', 'tax_date'))
 
 
 class TaxableMixin(object):
+    __slots__ = ()
 
     @property
     def taxable_lines(self):
@@ -1064,14 +1125,10 @@ class TaxableMixin(object):
             - the first element is the taxes applicable
             - the second element is the line unit price
             - the third element is the line quantity
+            - the forth element is the optional tax date
         """
         return []
 
-    @property
-    def currency(self):
-        "The currency used by the taxable object"
-        return None
-
     @property
     def tax_date(self):
         "Date to use when computing the tax"
@@ -1095,7 +1152,7 @@ class TaxableMixin(object):
         line['legal_notice'] = tax.legal_notice
         line['base'] = base
         line['amount'] = amount
-        line['tax'] = tax.id if tax else None
+        line['tax'] = tax.id
         line['account'] = getattr(tax, '%s_account' % type_).id
 
         return _TaxKey(**line)
@@ -1106,22 +1163,25 @@ class TaxableMixin(object):
         for taxline in taxes.values():
             taxline['amount'] = self.currency.round(taxline['amount'])
 
+    @fields.depends(methods=['_get_tax_context', '_round_taxes'])
     def _get_taxes(self):
         pool = Pool()
         Tax = pool.get('account.tax')
         Configuration = pool.get('account.configuration')
 
-        config = Configuration(1)
-        tax_rounding = config.tax_rounding
         taxes = {}
         with Transaction().set_context(self._get_tax_context()):
+            config = Configuration(1)
+            tax_rounding = config.get_multivalue('tax_rounding')
             taxable_lines = [_TaxableLine(*params)
                 for params in self.taxable_lines]
             for line in taxable_lines:
                 l_taxes = Tax.compute(Tax.browse(line.taxes), line.unit_price,
-                    line.quantity, self.tax_date)
+                    line.quantity, line.tax_date or self.tax_date)
                 for tax in l_taxes:
                     taxline = self._compute_tax_line(**tax)
+                    # Base must always be rounded per line as there will be one
+                    # tax line per taxable_lines
                     if self.currency:
                         taxline['base'] = self.currency.round(taxline['base'])
                     if taxline not in taxes:
@@ -1148,7 +1208,7 @@ class TaxLine(ModelSQL, ModelView):
             ('base', "Base"),
             ], "Type", required=True)
     tax = fields.Many2One('account.tax', 'Tax', select=True,
-        ondelete='RESTRICT',
+        ondelete='RESTRICT', required=True,
         domain=[
             ('company', '=', Eval('company', -1)),
             ],
@@ -1160,23 +1220,19 @@ class TaxLine(ModelSQL, ModelView):
 
     @classmethod
     def __setup__(cls):
-        super(TaxLine, cls).__setup__()
-        cls._error_messages.update({
-                'modify_closed_period': (
-                    'You can not add/modify tax lines in closed period "%s".'),
-                })
+        super().__setup__()
+        cls.__access__.add('move_line')
 
     @classmethod
     def __register__(cls, module_name):
         pool = Pool()
         Tax = pool.get('account.tax')
-        TableHandler = backend.get('TableHandler')
         transaction = Transaction()
         table = cls.__table__()
         tax = Tax.__table__()
 
         migrate_type = False
-        if TableHandler.table_exist(cls._table):
+        if backend.TableHandler.table_exist(cls._table):
             table_h = cls.__table_handler__(module_name)
             migrate_type = not table_h.column_exist('type')
 
@@ -1208,7 +1264,7 @@ class TaxLine(ModelSQL, ModelView):
                         & (table.code.in_(
                                 [invoice_base_code, credit_note_base_code]))))
 
-    @fields.depends('move_line')
+    @fields.depends('move_line', '_parent_move_line.currency_digits')
     def on_change_with_currency_digits(self, name=None):
         if self.move_line:
             return self.move_line.currency_digits
@@ -1255,8 +1311,9 @@ class TaxLine(ModelSQL, ModelView):
         for line in lines:
             period = line.period_checked
             if period and period.state != 'open':
-                cls.raise_user_error(
-                    'modify_closed_period', (period.rec_name,))
+                raise AccessError(
+                    gettext('account.msg_modify_tax_line_closed_period',
+                        period=period.rec_name))
 
 
 class TaxRuleTemplate(ModelSQL, ModelView):
@@ -1326,11 +1383,8 @@ class TaxRule(ModelSQL, ModelView):
         }
     name = fields.Char('Name', required=True, states=_states)
     kind = fields.Selection(KINDS, 'Kind', required=True, states=_states)
-    company = fields.Many2One('company.company', 'Company', required=True,
-        select=True, domain=[
-            ('id', If(Eval('context', {}).contains('company'), '=', '!='),
-                Eval('context', {}).get('company', -1)),
-            ])
+    company = fields.Many2One(
+        'company.company', "Company", required=True, select=True)
     lines = fields.One2Many('account.tax.rule.line', 'rule', 'Lines')
     template = fields.Many2One('account.tax.rule.template', 'Template')
     template_override = fields.Boolean("Override Template",
@@ -1407,6 +1461,8 @@ class TaxRuleLineTemplate(sequence_ordered(), ModelSQL, ModelView):
     __name__ = 'account.tax.rule.line.template'
     rule = fields.Many2One('account.tax.rule.template', 'Rule', required=True,
             ondelete='CASCADE')
+    start_date = fields.Date("Starting Date")
+    end_date = fields.Date("Ending Date")
     group = fields.Many2One('account.tax.group', 'Tax Group',
         ondelete='RESTRICT')
     origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
@@ -1418,8 +1474,8 @@ class TaxRuleLineTemplate(sequence_ordered(), ModelSQL, ModelView):
                 ('group', '=', None),
                 If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
                     ('group.kind', 'in', ['sale', 'both']),
-                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
-                        'purchase',
+                    If(Eval('_parent_rule', {}).get('kind', 'both')
+                        == 'purchase',
                         ('group.kind', 'in', ['purchase', 'both']),
                         ('group.kind', 'in', ['sale', 'purchase', 'both']))),
                 ],
@@ -1439,8 +1495,8 @@ class TaxRuleLineTemplate(sequence_ordered(), ModelSQL, ModelView):
                 ('group', '=', None),
                 If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
                     ('group.kind', 'in', ['sale', 'both']),
-                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
-                        'purchase',
+                    If(Eval('_parent_rule', {}).get('kind', 'both')
+                        == 'purchase',
                         ('group.kind', 'in', ['purchase', 'both']),
                         ('group.kind', 'in', ['sale', 'purchase', 'both']))),
                 ],
@@ -1451,6 +1507,7 @@ class TaxRuleLineTemplate(sequence_ordered(), ModelSQL, ModelView):
     @classmethod
     def __setup__(cls):
         super(TaxRuleLineTemplate, cls).__setup__()
+        cls.__access__.add('rule')
         cls._order.insert(1, ('rule', 'ASC'))
 
     def _get_tax_rule_line_value(self, rule_line=None):
@@ -1458,6 +1515,10 @@ class TaxRuleLineTemplate(sequence_ordered(), ModelSQL, ModelView):
         Set values for tax rule line creation.
         '''
         res = {}
+        if not rule_line or rule_line.start_date != self.start_date:
+            res['start_date'] = self.start_date
+        if not rule_line or rule_line.end_date != self.end_date:
+            res['end_date'] = self.end_date
         if not rule_line or rule_line.group != self.group:
             res['group'] = self.group.id if self.group else None
         if not rule_line or rule_line.sequence != self.sequence:
@@ -1521,6 +1582,8 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         }
     rule = fields.Many2One('account.tax.rule', 'Rule', required=True,
             select=True, ondelete='CASCADE', states=_states)
+    start_date = fields.Date("Starting Date")
+    end_date = fields.Date("Ending Date")
     group = fields.Many2One('account.tax.group', 'Tax Group',
         ondelete='RESTRICT', states=_states)
     origin_tax = fields.Many2One('account.tax', 'Original Tax',
@@ -1532,8 +1595,8 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
                 ('group', '=', None),
                 If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
                     ('group.kind', 'in', ['sale', 'both']),
-                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
-                        'purchase',
+                    If(Eval('_parent_rule', {}).get('kind', 'both')
+                        == 'purchase',
                         ('group.kind', 'in', ['purchase', 'both']),
                         ('group.kind', 'in', ['sale', 'purchase', 'both']))),
                 ],
@@ -1553,8 +1616,8 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
                 ('group', '=', None),
                 If(Eval('_parent_rule', {}).get('kind', 'both') == 'sale',
                     ('group.kind', 'in', ['sale', 'both']),
-                    If(Eval('_parent_rule', {}).get('kind', 'both') ==
-                        'purchase',
+                    If(Eval('_parent_rule', {}).get('kind', 'both')
+                        == 'purchase',
                         ('group.kind', 'in', ['purchase', 'both']),
                         ('group.kind', 'in', ['sale', 'purchase', 'both']))),
                 ],
@@ -1574,6 +1637,7 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
     @classmethod
     def __setup__(cls):
         super(TaxRuleLine, cls).__setup__()
+        cls.__access__.add('rule')
         cls._order.insert(1, ('rule', 'ASC'))
 
     @classmethod
@@ -1586,9 +1650,19 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
         return super(TaxRuleLine, cls).copy(lines, default=default)
 
     def match(self, pattern):
+        pool = Pool()
+        Date = pool.get('ir.date')
+        with Transaction().set_context(company=self.rule.company.id):
+            today = Date.today()
+        pattern = pattern.copy()
         if 'group' in pattern and not self.group:
             if pattern['group']:
                 return False
+        date = pattern.pop('date', None) or today
+        if self.start_date and date < self.start_date:
+            return False
+        if self.end_date and date > self.end_date:
+            return False
         return super(TaxRuleLine, self).match(pattern)
 
     def get_taxes(self, origin_tax):
@@ -1632,8 +1706,8 @@ class TaxRuleLine(sequence_ordered(), ModelSQL, ModelView, MatchMixin):
                     vals['rule'] = template2rule[line.template.rule.id]
                 if line.origin_tax:
                     if line.template.origin_tax:
-                        if (line.origin_tax.id !=
-                                template2tax[line.template.origin_tax.id]):
+                        if (line.origin_tax.id
+                                != template2tax[line.template.origin_tax.id]):
                             vals['origin_tax'] = template2tax[
                                 line.template.origin_tax.id]
                     else:
@@ -1664,13 +1738,13 @@ class OpenTaxCode(Wizard):
 
     def do_open_(self, action):
         pool = Pool()
-        TaxCode = pool.get('account.tax.code')
         Tax = pool.get('account.tax')
-        tax_code = TaxCode(Transaction().context['active_id'])
-        if tax_code.lines:
-            domain = ['OR'] + [l._line_domain for l in tax_code.lines]
+        if self.record.lines:
+            domain = ['OR'] + [l._line_domain for l in self.record.lines]
         else:
             domain = ('id', '=', None)
+        if self.record:
+            action['name'] += ' (%s)' % self.record.rec_name
         action['pyson_domain'] = PYSONEncoder().encode([
                 Tax._amount_domain(),
                 domain,
@@ -1687,10 +1761,11 @@ class TestTax(Wizard):
         [Button('Close', 'end', 'tryton-close', default=True)])
 
     def default_test(self, fields):
-        context = Transaction().context
         default = {}
-        if context['active_model'] == 'account.tax':
-            default['taxes'] = context['active_ids']
+        if (self.model
+                and self.model.__name__ == 'account.tax'
+                and self.records):
+            default['taxes'] = list(map(int, self.records))
         return default
 
 
@@ -1729,7 +1804,7 @@ class TestTaxView(ModelView, TaxableMixin):
 
     @property
     def taxable_lines(self):
-        return [(self.taxes, self.unit_price, self.quantity)]
+        return [(self.taxes, self.unit_price, self.quantity, self.tax_date)]
 
     @fields.depends(
         'tax_date', 'taxes', 'unit_price', 'quantity', 'currency', 'result')
@@ -1739,6 +1814,7 @@ class TestTaxView(ModelView, TaxableMixin):
         result = []
         if all([self.tax_date, self.unit_price, self.quantity, self.currency]):
             for taxline in self._get_taxes():
+                del taxline['manual']
                 result.append(Result(**taxline))
         self.result = result
         return self._changed_values.get('result', [])
@@ -1749,10 +1825,7 @@ class TestTaxViewResult(ModelView):
     __name__ = 'account.tax.test.result'
     tax = fields.Many2One('account.tax', "Tax")
     description = fields.Char("Description")
+    legal_notice = fields.Char("Legal Notice")
     account = fields.Many2One('account.account', "Account")
     base = fields.Numeric("Base")
-    base_code = fields.Many2One('account.tax.code', "Base Code")
-    base_sign = fields.Numeric("Base Sign", digits=(2, 0))
     amount = fields.Numeric("Amount")
-    tax_code = fields.Many2One('account.tax.code', "Tax Code")
-    tax_sign = fields.Numeric("Tax Sign", digits=(2, 0))
