@@ -205,6 +205,17 @@ class AccountTestCase(
             update_chart.start.account = root
             update_chart.transition_update()
 
+    @with_transaction()
+    def test_account_chart_many_companies(self):
+        "Test creation of chart of accounts for many companies"
+        company1 = create_company()
+        with set_company(company1):
+            create_chart(company1, tax=True)
+
+        company2 = create_company()
+        with set_company(company2):
+            create_chart(company2, tax=True)
+
     @with_transaction()
     def test_fiscalyear(self):
         'Test fiscalyear'
@@ -1531,6 +1542,7 @@ class AccountTestCase(
         AccountTemplate = pool.get('account.account.template')
         TaxTemplate = pool.get('account.tax.template')
         TaxCodeTemplate = pool.get('account.tax.code.template')
+        TaxCodeTemplateLine = pool.get('account.tax.code.line.template')
         UpdateChart = pool.get('account.update_chart', type='wizard')
         Type = pool.get('account.account.type')
         Account = pool.get('account.account')
@@ -1589,7 +1601,7 @@ class AccountTestCase(
             tax_code.template_override = True
             tax_code.save()
 
-            template_tax_code_line, = TaxCodeLine.search([], limit=1)
+            template_tax_code_line, = TaxCodeTemplateLine.search([], limit=1)
             tax_code_line, = TaxCodeLine.search(
                 [('template', '=', template_tax_code_line.id)])
             tax_code_line.template_override = True
