@@ -9,13 +9,14 @@ from proteus import Model, Wizard
 from trytond.modules.company.tests.tools import get_company
 
 __all__ = ['create_fiscalyear', 'create_chart', 'get_accounts',
-    'create_tax', 'set_tax_code']
+    'create_tax', 'create_tax_code']
 
 
 def create_fiscalyear(company=None, today=None, config=None):
     "Create a fiscal year for the company on today"
     FiscalYear = Model.get('account.fiscalyear', config=config)
     Sequence = Model.get('ir.sequence', config=config)
+    SequenceType = Model.get('ir.sequence.type', config=config)
 
     if not company:
         company = get_company()
@@ -28,23 +29,30 @@ def create_fiscalyear(company=None, today=None, config=None):
     fiscalyear.end_date = today + relativedelta(month=12, day=31)
     fiscalyear.company = company
 
-    post_move_sequence = Sequence(name=str(today.year), code='account.move',
+    sequence_type, = SequenceType.find(
+        [('name', '=', "Account Move")], limit=1)
+    post_move_sequence = Sequence(
+        name=str(today.year),
+        sequence_type=sequence_type,
         company=company)
     post_move_sequence.save()
     fiscalyear.post_move_sequence = post_move_sequence
     return fiscalyear
 
 
-def create_chart(company=None, config=None):
+def create_chart(
+        company=None, chart='account.account_template_root_en', config=None):
     "Create chart of accounts"
     AccountTemplate = Model.get('account.account.template', config=config)
     ModelData = Model.get('ir.model.data')
 
     if not company:
         company = get_company()
+
+    module, xml_id = chart.split('.')
     data, = ModelData.find([
-            ('module', '=', 'account'),
-            ('fs_id', '=', 'account_template_root_en'),
+            ('module', '=', module),
+            ('fs_id', '=', xml_id),
             ], limit=1)
 
     account_template = AccountTemplate(data.db_id)
@@ -57,8 +65,10 @@ def create_chart(company=None, config=None):
 
     accounts = get_accounts(company, config=config)
 
-    create_chart.form.account_receivable = accounts['receivable']
-    create_chart.form.account_payable = accounts['payable']
+    if accounts['receivable'].party_required:
+        create_chart.form.account_receivable = accounts['receivable']
+    if accounts['payable'].party_required:
+        create_chart.form.account_payable = accounts['payable']
     create_chart.execute('create_properties')
     return create_chart
 
@@ -70,23 +80,29 @@ def get_accounts(company=None, config=None):
     if not company:
         company = get_company()
 
-    accounts = Account.find([
-            ('kind', 'in', ['receivable', 'payable', 'revenue', 'expense']),
-            ('company', '=', company.id),
-            ])
-    accounts = {a.kind: a for a in accounts}
-    cash, = Account.find([
-            ('kind', '=', 'other'),
-            ('company', '=', company.id),
-            ('name', '=', 'Main Cash'),
-            ])
-    accounts['cash'] = cash
-    tax, = Account.find([
-            ('kind', '=', 'other'),
-            ('company', '=', company.id),
-            ('name', '=', 'Main Tax'),
-            ])
-    accounts['tax'] = tax
+    accounts = {}
+    for type in ['receivable', 'payable', 'revenue', 'expense']:
+        try:
+            accounts[type], = Account.find([
+                    ('type.%s' % type, '=', True),
+                    ('company', '=', company.id),
+                    ], limit=1)
+        except ValueError:
+            pass
+    try:
+        accounts['cash'], = Account.find([
+                ('company', '=', company.id),
+                ('name', '=', 'Main Cash'),
+                ], limit=1)
+    except ValueError:
+        pass
+    try:
+        accounts['tax'], = Account.find([
+                ('company', '=', company.id),
+                ('name', '=', 'Main Tax'),
+                ], limit=1)
+    except ValueError:
+        pass
     return accounts
 
 
@@ -109,21 +125,16 @@ def create_tax(rate, company=None, config=None):
     return tax
 
 
-def set_tax_code(tax, config=None):
-    "Set code on tax"
+def create_tax_code(
+        tax, amount='tax', type='invoice', operator='+', config=None):
+    "Create a tax code for the tax"
     TaxCode = Model.get('account.tax.code', config=config)
-    invoice_base_code = TaxCode(name='Invoice Base')
-    invoice_base_code.save()
-    invoice_tax_code = TaxCode(name='Invoice Tax')
-    invoice_tax_code.save()
-    credit_note_base_code = TaxCode(name='Credit Note Base')
-    credit_note_base_code.save()
-    credit_note_tax_code = TaxCode(name='Credit Note Tax')
-    credit_note_tax_code.save()
 
-    tax.invoice_base_code = invoice_base_code
-    tax.invoice_tax_code = invoice_tax_code
-    tax.credit_note_base_code = credit_note_base_code
-    tax.credit_note_tax_code = credit_note_tax_code
-
-    return tax
+    tax_code = TaxCode(name="Tax Code %s" % tax.name)
+    tax_code.company = tax.company
+    line = tax_code.lines.new()
+    line.operator = '+'
+    line.tax = tax
+    line.amount = amount
+    line.type = type
+    return tax_code
